import re

from varst.utils.rst_file import RstFile


class Substitution:

    def __init__(self, rst_file: RstFile):
        self.rst_file = rst_file

    def find(self, name) -> str:
        """Find substitution text by name.

        Args:
            name: The string value of substitution name.
        Returns:
            substitution_text: The string value of substitution text.
        Raises:
            KeyError: If substitution is not in file.

        """
        pattern = re.compile(
            fr"""
                \.\.[ ]+          # explicit markup start
                \|                # substitution indicator
                ({name})\|        # substitution name
            """, re.VERBOSE,
        )

        for content in self.rst_file.contents:
            if pattern.match(content):
                return content
        raise KeyError(name)

    def update(self, name, value):
        """Update substitution text with name to value.

        Args:
            name: The string value of substitution name.
            value: The string value of substitution text.

        """
        origin = self.find(name)
        new = substitution_text(name, value)

        origin_idx = self.rst_file.contents.index(origin)
        self.rst_file.contents.pop(origin_idx)
        self.rst_file.contents.insert(origin_idx, new + "\n")


def substitution_text(name, value) -> str:
    return f'.. |{name}| replace:: {value}'



from typing import Optional
from typing import Sequence

from varst import application


def main(argv: Optional[Sequence[str]] = None):
    """Main function to run application

        Args:
            argv: Arguments vector

    """
    app = application.Application()
    app.run(argv)



import argparse
import os
import re
from argparse import ArgumentParser
from typing import Dict
from typing import List
from typing import Optional
from typing import Sequence

from varst import supported


class Parser:
    """Parser class that parse arguments from cli"""

    _parser = ArgumentParser()

    input_file: str = ""
    output_file: str = ""
    variables: Dict[str, str] = {}

    def __init__(self) -> None:
        self._parser.add_argument(
            "name=value", nargs="*",
            help="name-value pairs of substitutions",
            type=_pattern_type,
        )
        self._parser.add_argument(
            "-i",
            "--input",
            type=_file_type,
            help="rst file path as input",
            default="./README.rst",
        )
        self._parser.add_argument(
            "-o",
            "--output",
            type=_file_type,
            help="rst file path as output",
            default="./README.rst",
        )

    def parse(self, argv: Optional[Sequence[str]]) -> None:
        """Parse the arguments from argv.
        Parsed values are updated in the attributes of the parser.

        Args:
            argv: Arguments vector

        """
        args = self._parser.parse_args(argv)
        arg_dict = vars(args)

        self.input_file = arg_dict['input']
        self.output_file = arg_dict['output']
        self.variables = _parse_kv(arg_dict['name=value'])


_VARIABLE_PATTERN = re.compile(r"[^=]+=[^=]+")


def _pattern_type(arg_value: str, pat=_VARIABLE_PATTERN):
    """Verify that the arg_value fully matches the pattern.

    Args:
        arg_value: The string value to check pattern.
        pat: The pattern to match value.
    Returns:
        str: Returns arg_value that matched with pattern.

    """
    if not pat.fullmatch(arg_value):
        raise argparse.ArgumentTypeError(f"invalid pattern: {pat.pattern}")
    return arg_value


def _file_type(file_name: str):
    ext = os.path.splitext(file_name)[1][1:]
    if ext.lower() not in supported:
        raise argparse.ArgumentTypeError(
            f"file extension must be in {supported}",
        )
    return file_name


def _parse_kv(variables: List[str]) -> Dict[str, str]:
    """Parse ``key-value`` pair from variables.

    Args:
        variables: The string list to be parsed.
    Returns:
        Dict[str, str]: key-value pair

    """
    result: Dict[str, str] = {}

    for variable in variables:
        kv = variable.split("=")
        result[kv.pop()] = kv.pop()

    return result



from typing import List


class RstFile:
    """Class to handle rst file

    Args:
        src: The source path to open rst file.
    """

    def __init__(self, src: str):
        with open(src, encoding='utf-8') as f:
            self._contents = f.readlines()

    @property
    def contents(self) -> List[str]:
        return self._contents

    def save(self, dest: str) -> None:
        """Save contents to the destination path.

        Args:
            dest: The destination path to save rst file.

        """
        with open(dest, encoding='utf-8', mode='w') as f:
            f.write("".join(self.contents))



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['varst', 'varst.utils']

package_data = \
{'': ['*']}

entry_points = \
{'console_scripts': ['varst = varst.cli:main']}

setup_kwargs = {
    'name': 'varst',
    'version': '1.1.0',
    'description': 'Replace substitutions in rst files with variables.',
    'long_description': '==============================\nvarST(var ➡️ reStructuredText)\n==============================\n\n|PyPI version| |pre-commit.ci status| |GitHub Workflow Status| |Documentation Status|\n\nReplace substitutions in rst files with variables.\n\nInstallation\n------------\n\n.. code:: bash\n\n   $ pip install varst\n\nUsage\n-----\n\n.. code:: bash\n\n   $ varst [-i INPUT] [-o OUTPUT] [name=value ...]\n\nLicense\n-------\n\n`MIT\nLicense <https://github.com/junghoon-vans/varst/blob/main/LICENSE>`__\n\n\n.. |PyPI version| image:: https://img.shields.io/pypi/v/varst\n   :target: https://pypi.org/project/varst/\n.. |pre-commit.ci status| image:: https://results.pre-commit.ci/badge/github/junghoon-vans/varst/main.svg\n   :target: https://results.pre-commit.ci/latest/github/junghoon-vans/varst/main\n.. |GitHub Workflow Status| image:: https://img.shields.io/github/workflow/status/junghoon-vans/varst/Upload%20Python%20Package\n.. |Documentation Status| image:: https://readthedocs.org/projects/varst/badge/?version=latest\n    :target: https://varst.readthedocs.io/en/latest/?badge=latest\n',
    'author': 'Jeonghun-Ban',
    'author_email': 'junghoon.ban@gmail.com',
    'maintainer': 'Jeonghun-Ban',
    'maintainer_email': 'junghoon.ban@gmail.com',
    'url': 'https://github.com/junghoon-vans/varst',
    'packages': packages,
    'package_data': package_data,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



"""Module containing the application class"""
from typing import Optional
from typing import Sequence

from varst.utils.parser import Parser
from varst.utils.rst_file import RstFile
from varst.utils.substitution import Substitution


class Application:
    """Application class"""

    def __init__(self) -> None:
        self.parser = Parser()

    def run(self, argv: Optional[Sequence[str]]):
        """Run application to replace substitutions.

            Args:
                argv: Arguments vector

        """
        self.parser.parse(argv)

        rst_file = RstFile(src=self.parser.input_file)
        substitution = Substitution(rst_file)
        for k, v in self.parser.variables.items():
            substitution.update(k, v)
        rst_file.save(dest=self.parser.output_file)
